package com.gloomyer.diff_update;

import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import com.google.gson.Gson;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = "MainActivity";
    private static final String BASEURL = "http://192.168.3.213:8090/";
    private static final String requestUrl = BASEURL + "DiffServlet";
    private Handler mHander;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mHander = new Handler() {
            @Override
            public void handleMessage(Message msg) {
                if (msg.what == 0) {
                    downloadPatch((UpdateBean) msg.obj);
                } else if (msg.what == 1) {
                    ApkUtils.installApk(MainActivity.this, (String) msg.obj);
                }
            }

        };
    }

    private void downloadPatch(final UpdateBean info) {
        Log.e(TAG, info.toString());
        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    URL url = new URL(info.getUrl());
                    HttpURLConnection connection = (HttpURLConnection) url.openConnection();
                    connection.setDoOutput(true);
                    connection.setDoInput(true);
                    connection.setUseCaches(false);
                    connection.setConnectTimeout(60000);
                    connection.setReadTimeout(60000);

                    File patchFile = Environment.getExternalStorageDirectory();
                    patchFile = new File(patchFile, "patch_" + System.currentTimeMillis() + ".patch");

                    connection.connect();
                    if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
                        InputStream is = connection.getInputStream();
                        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(patchFile));

                        int len;
                        byte buffer[] = new byte[512];
                        while ((len = is.read(buffer)) > 0) {
                            bos.write(buffer, 0, len);
                        }

                        bos.flush();
                        bos.close();
                        is.close();
                    }

                    //合并文件
                    File newApkFile = Environment.getExternalStorageDirectory();
                    newApkFile = new File(newApkFile, "apkfile_" + System.currentTimeMillis() + ".apk");
                    JniUtils.merge(ApkUtils.getOldPath(MainActivity.this, getPackageName()),
                            patchFile.getAbsolutePath(),
                            newApkFile.getAbsolutePath());

                    mHander.obtainMessage(1, newApkFile.getAbsolutePath()).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        }).start();
    }

    public void update(View v) {

        if (true) {
            Toast.makeText(this, "已经是最新的了!", Toast.LENGTH_SHORT).show();
            return;
        }

        new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    URL url = new URL(requestUrl);
                    HttpURLConnection connection = (HttpURLConnection) url.openConnection();
                    connection.setRequestMethod("GET");
                    connection.setRequestProperty("Charset", "UTF-8");
                    connection.setRequestProperty("Content-Type", "text/xml; charset=UTF-8");
                    connection.connect();
                    if (connection.getResponseCode() == 200) {
                        InputStream is = connection.getInputStream();
                        int len;
                        byte buffer[] = new byte[512];
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        if ((len = is.read(buffer)) > 0) {
                            baos.write(buffer, 0, len);
                        }
                        baos.flush();

                        String json = new String(baos.toByteArray());
                        UpdateBean info = new Gson().fromJson(json, UpdateBean.class);
                        baos.close();
                        is.close();


                        mHander.obtainMessage(0, info).sendToTarget();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }).start();
    }
}
